'use strict';

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _cssbeautify = require('cssbeautify');

var _cssbeautify2 = _interopRequireDefault(_cssbeautify);

var _validateOpts = require('./validateOpts');

var _makeContainerClass = require('./makeContainerClass');

var _makeContainerMaxWidths = require('./makeContainerMaxWidths');

var _makeRowClass = require('./makeRowClass');

var _makeNoGuttersUtil = require('./makeNoGuttersUtil');

var _columns = require('./columns');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const makeGrid = (pluginOpts = {}) => new Promise(async (resolve, reject) => {
    try {
        const opts = await (0, _validateOpts.validateOpts)(pluginOpts);
        resolve((0, _cssbeautify2.default)(`

${(0, _makeContainerClass.makeContainerClass)(opts)}
${(0, _makeContainerMaxWidths.makeContainerMaxWidths)(opts)}
${(0, _makeRowClass.makeRowClass)(opts)}
${(0, _makeNoGuttersUtil.makeNoGuttersUtil)()}
${(0, _columns.makeGridColumns)(opts)}

      `, {
            indent: '  ',
            autosemicolon: true
        }));
    } catch (error) {
        reject(error);
    }
});

const bootstrap4Grid = (opts = {}) => (root, result) => new Promise((resolve, reject) => {
    const atRuleCount = root.toString().split('@bootstrap-4-grid').length - 1;

    if (atRuleCount === 0) {
        resolve();
        return;
    }

    let steps = 0;
    root.walkAtRules('bootstrap-4-grid', async rule => {
        try {
            const css = await makeGrid(opts);
            rule.before(css);
            rule.remove();
            steps += 1;
        } catch (error) {
            steps += 1;
            result.warn(err);
        }
    });

    const completionInterval = setInterval(() => {
        if (steps === atRuleCount) {
            clearInterval(completionInterval);
            resolve();
        }
    }, 1000);
});

module.exports = _postcss2.default.plugin('bootstrap-4-grid', bootstrap4Grid);